
/*

  Select_By_Normal.ls
  v1.1.2
  Matt Gorner

  v1.1.1 - 02.07.2010 - Fixed issue where it would leave out 1 polygon
  
  v1.1.2 - 08.07.2010 - Added ability to limit to polygons with the same surface only
  
  ----------------------------------------------------------------------------------------------------------------------

  Thanks to Przemyslaw Gruchala (aka Sensei) for help with the math in this little script!

  Selects polygons that have the same normal direction.

  1) Select a polygon (you can select more than one, but it will only use the FIRST polygon as the normal to select by)
  
  2) Choose whether to limit to same surface

  3) Click 'Update' for live preview, or click okay to select and close the script

  The script will now select all polygons whose normal is within the tolerance specified of the original selected normal poly,
  as well as the same surface if the option was set

*/

@version 2.2
@warnings
@script modeler
@name "Select By Normal"

// Globals

// Check tolerance in degrees
tolerance = 0.005;
skipped = 0;
already_updated = FALSE;

selected_poly_normal;
selected_poly_surface;
same_surf = 0;

main
{
	// Get previous tolerance setting if the user specified one in the Modeler session
	tolerance = recall("SBN_tolerance", tolerance);
	same_surf = recall("SBN_same_surf", same_surf);

	// Group our undos because each editbegin() creates one, but there is nothing to undo in this script, ideally be nice to be able to turn it off temporarily
	undogroupbegin();

	// Get only those polys selected by the user
	selmode(DIRECT);

	// Store the polygon info array
	totalpolys = polycount();

	// Check we have some polygons to work with
	if (totalpolys[1] == 0)
		error("You need to select a polygon you want to match to");

   // Go to 'edit' mode to access polygon data in Modeler
	editbegin();

	// Store the first selected polygon normal and it's surface
	selected_poly_normal	= polynormal(polygons[1]);
	selected_poly_surface	= polygons[1].surface;

	// Need to end here so we can use the selpolyon command, as it doesn't work inside edit mode
	editend();

	// User interface layout variables
	req_width		= 255;
	req_height	  	= 115;
	layout_x	  	= 4;
	layout_y	  	= 10;
	layout_width	= 150;
	layout_height	= 19;
	
	slot_offset	  	= 22;
	spacer		  	= 5;
	text_offset 	= 90;

	row_number		= 0;


	// Setup interface . . .
 	reqbegin("Select by Normal ...");
	reqsize(req_width, req_height);

	// Initialise gadgets
	gadget_tolerance = ctlnumber("Tolerance (Deg)", tolerance);
	ctlposition(gadget_tolerance, layout_x, layout_y + (layout_height * row_number) + (spacer * row_number), layout_width + text_offset, layout_height, text_offset);
	ctlrefresh(gadget_tolerance, "update_tolerance");
	
	row_number ++ ;

	gadget_same_surf = ctlcheckbox("Same Surface Only?", same_surf);
	ctlposition(gadget_same_surf, layout_x + text_offset, layout_y + (layout_height * row_number) + (spacer * row_number), layout_width, layout_height);
	
	row_number ++ ;

	gadget_update = ctlbutton("Update", layout_width, "update_selection");
	ctlposition(gadget_update, layout_x + text_offset, layout_y + (layout_height * row_number) + (spacer * row_number), layout_width, layout_height);

	// Show interface and gadgets (exit script if it fails)
	return if !reqpost();

		// Grab user input
		tolerance = getvalue(gadget_tolerance);
		same_surf = getvalue(gadget_same_surf);

	// Close requester
	reqend();

	// Store tolerance for next time (in one active Modeler session only)
	store("SBN_tolerance", tolerance);
	store("SBN_same_surf", same_surf);

	// Don't update if the user already pressed update after changing the tolerance value
	if(!already_updated)
		update_selection();

	// If there were skipped polys, let the user know
	if(skipped)
		info("Couldn't calculate normals for ", skipped + 1, " polygons"); // Need to add 1, otherwise it's always 1 less than it should be, dunno why though!

	// End undo grouping
	undogroupend();
}

// Callback function for tolerance gadget
update_tolerance: value
{
	// Get tolerance value from gadget (this callback is passed the gadget value when it's called)
	tolerance = value;

	// Unless the user typed something different (could have clicked, not changed and returned out)
	if(value != tolerance)
		// The selection will need to be shown to the user
		already_updated = FALSE;
}

// Callback function for update button gadget
update_selection
{
	// Clear existing selection, change selection mode to work on everything
	selpolygon(CLEAR);
	selmode(USER);

	// Get the total number of polys again (because we cleared the user selection)
	totalpolys = polycount();

	// We need to access polygon data again
	editbegin();

	// Don't need these lines anymore but have left in there for learning purposes
	// Define min / max check tolerance vectors
	// minv = <selected_poly_normal.x - tolerance, selected_poly_normal.y - tolerance, selected_poly_normal.z - tolerance>;
	// maxv = <selected_poly_normal.x + tolerance, selected_poly_normal.y + tolerance, selected_poly_normal.z + tolerance>;

	// Loop through all polys to check their normal direction based on the one we stored earlier
	for (loop = 1; loop < totalpolys[1] + 1; loop++)
	{
		// A valid normal?  Not checking here was causing an error
		if(polynormal(polygons[loop]) == NULL)
		{
			// No, we're skipping then!
			skipped ++;
		}else
		{
			// Same surfaces only?
			if(same_surf)
			{
				// Same surface?
				if(polygons[loop].surface == selected_poly_surface)
				{
					// Apparently so, now check poly normal against our stored one
					dot = dot3d( polynormal(polygons[loop]), selected_poly_normal );
					if( dot >= cos( rad( tolerance) ) )
					{
					  // Within range, add current poly ID to our selection array for later
					  poly_IDs_to_select += polygons[loop];
					}
				}
			}else
			{
				// Don't need this line anymore but have left in there for learning purposes
				//if( (polynormal(polygons[loop]) >= minv) && (polynormal(polygons[loop]) <= maxv) )
	
				// Check current poly normal against our stored one
				dot = dot3d( polynormal(polygons[loop]), selected_poly_normal );

				if( dot >= cos( rad( tolerance) ) )
				{
				  // Add current poly ID to our selection array for later
				  poly_IDs_to_select += polygons[loop];
				}
			}
		}
	}

	// Finished accessing polygon data, end edit mode
	editend();

	// Select polys that we stored (the ones identified as having the same normal direction)
	if( poly_IDs_to_select )
		 selpolygon(SET, POLYID, poly_IDs_to_select);

	// We have updated the selection and shown it to the user
	already_updated = TRUE;

}